@@ -24,7 +24,6 @@
 #include <mcrypt_int.h>
 #include <xmalloc.h>
 #include <extra.h>
-#include <locks.h>
 #include <keys.h>
 #include <random.h>
 #include <gaa.h>
@@ -32,12 +31,6 @@
 #include <environ.h>
 #include <popen.h>
 
-#ifdef HAVE_FTIME
-# include <sys/timeb.h>
-#else
-# include <time.h>
-#endif
-
 extern char tmperr[];
 extern unsigned int stream_flag;
 extern char *keymode;
@@ -46,7 +39,6 @@ extern char *algorithm;
 extern char *hash;
 extern int keysize;
 extern int nodelete;
-extern int nolock;
 extern int noiv;
 extern int hash_algorithm;
 extern int noecho;
@@ -61,12 +53,11 @@ extern int gzipflag;
 extern int bzipflag;
 #endif
 extern int flush;
-extern int timer;
 extern char *outfile, *tmpc;
 extern char *algorithms_directory;
 extern char *modes_directory;
 
-static void show_stats(time_t ms_time, size_t file_size);
+int make_mult(int num, int d);
 
 
 /* General Encryption 
@@ -75,6 +66,10 @@ static void show_stats(time_t ms_time, size_t file_size);
  * code. This is a mess.
  */
 
+/* Reading from now on offers nothing to the reader except
+ * of a brain damage.
+ */
+
 int
 encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 {
@@ -83,7 +78,7 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
    word32 *keyword;
    int tc, enc_bytes;
    MHASH tm = 0;
-   int i = 0, how = 0, clen;
+   int how = 0, clen;
    unsigned int len = 0;
    FILE *FROMF;
    int keylen, salt_size;
@@ -95,13 +90,6 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
    word32 *IV = NULL;
    word32 *salt = NULL;
    int hash_size = mhash_get_block_size(hash_algorithm);
-#ifdef HAVE_FTIME
-   struct timeb start_time;
-   struct timeb end_time;
-#else
-   time_t start_time;
-   time_t end_time;
-#endif
    size_t file_sum = 0;
 
    word8 *ciphertext, *ciphertext_pad = NULL;
@@ -123,32 +111,24 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 
    ciphertext = _mcrypt_malloc(blocksize * BUFFER_SIZE);
 
+   /* Generate the SALT
+    */
    if (bare_flag == FALSE) {
       salt = _mcrypt_malloc(SALT_SIZE);	/* 20 bytes salt */
 
       /* Fill the salt with random data */
-      for (i = 0; i < SALT_SIZE / sizeof(word32); i++) {
-	 if (real_random_flag == TRUE) {
+      mcrypt_randomize( salt, SALT_SIZE, real_random_flag==TRUE?1:0);
+   }
 
-	    salt[i] = SRAND32;
-	 } else {
-	    salt[i] = RAND32;
-	 }
-      }
-   }
+
+   /* Generate the IV 
+    */
    IV = _secure_mcrypt_malloc(mcrypt_enc_get_iv_size(td));
-/* Random number generator */
-   for (i = 0; i < mcrypt_enc_get_iv_size(td) / sizeof(word32); i++) {
-      if (noiv == FALSE) {
-	 if (real_random_flag == TRUE) {
-	    IV[i] = SRAND32;
-	 } else {
-	    IV[i] = RAND32;
-	 }
-      } else {
-	 IV[i] = 0;
-      }
-   }
+   if (noiv==FALSE)
+	   mcrypt_randomize( IV, mcrypt_enc_get_iv_size(td), real_random_flag==TRUE?1:0);
+   else
+   	   memset( IV, 0, mcrypt_enc_get_iv_size(td));
+
 
 #ifdef DEBUG
    fprintf(stderr, "IV: ");
@@ -171,22 +151,16 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 	    perror("fopen");
 	    return 1;
 	 }
-	 if (read_lock(fileno((FILE *) FROMF)) == -1) {
-	    err_crit("Unable to lock file. Try --nolock\n");
-	    return 1;
-	 }
 
 	 TOF = fopen(tofile, "wb");
 	 if (TOF == NULL) {
 	    perror("fopen");
 	    return -1;
 	 }
-	 if (write_lock(fileno((FILE *) TOF)) == -1) {
-	    err_crit("Unable to lock file. Try --nolock\n");
-	    return -1;
-	 }
 #ifdef ZIP
       } else {
+         err_info( "Output file will be compressed.\n");
+
 	 /* command if using gzip */
 	 clen = 5;
 	 if (fromfile != NULL)
@@ -222,11 +196,6 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 	    return 1;
 	 }
 
-	 if (stream_flag == FALSE
-	     && write_lock(fileno((FILE *) TOF)) == -1) {
-	    err_crit("Unable to lock file. Try --nolock\n");
-	    return -1;
-	 }
       }
 #endif
    }
@@ -265,18 +234,7 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
    if (flush == TRUE)
       fflush(TOF);
 
-   if (quiet == FALSE)
-      fprintf(stderr,
-	      _
-	      ("Algorithm: %s\nKey size: %d\nMode: %s\nKeyword mode: %s\nDigest: %s\n"),
-	      algorithm, keysize, mode, keymode, hash);
 
-   if (key == NULL) {
-      if (stream_flag == FALSE) {
-	 sprintf(tmperr, _("File: %s\n"), fromfile);
-	 err_warn(tmperr);
-      }
-   }
 /* Get keyword */
    if (key == NULL) {
       keyword =
@@ -293,12 +251,7 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
       return 1;
    }
 
-#ifdef HAVE_FTIME
-   ftime(&start_time);
-#else
-   start_time = time(0);
-#endif
-
+   _mcrypt_start_timer();
    tc = mcrypt_generic_init(td, keyword, len, IV);
    if (tc < 0) {
       mcrypt_perror(tc);
@@ -408,26 +361,16 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 	 return 1;
       }
    }
-#ifdef HAVE_FTIME
-   ftime(&end_time);
-#else
-   end_time = time(0);
-#endif
 
    /* if bare_flag==FALSE */
    /* End of copy */
-   /* close and unlock files */
+   /* close files */
    fflush(TOF);
 
    if (stream_flag == FALSE) {
       fflush(FROMF);
       fflush(TOF);
 
-#ifdef ZIP
-      if (gzipflag == FALSE && bzipflag == FALSE)
-#endif
-	 unlock(fileno((FILE *) FROMF));
-      unlock(fileno((FILE *) TOF));
    }
    fclose(TOF);
 
@@ -443,6 +386,7 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 /* Ready */
 
    mcrypt_generic_end(td);
+   _mcrypt_end_timer();
 
    if (bare_flag == FALSE) {
       _mcrypt_free(salt);
@@ -451,31 +395,9 @@ encrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
    _secure_mcrypt_free(keyword, keylen);
    _mcrypt_free(ciphertext);
 
+   print_enc_info( fromfile, tofile);
 
-   if (timer != FALSE) {
-#ifdef HAVE_FTIME
-      end_time.time *= 1000;
-      end_time.time += end_time.millitm;
-
-      start_time.time *= 1000;
-      start_time.time += start_time.millitm;
-
-      end_time.time -= start_time.time;
-
-      start_time.time = end_time.time;
-
-      end_time.millitm = end_time.time % 1000;
-
-      show_stats(end_time.time, file_sum);
-
-#else
-      end_time -= start_time;
-
-      show_stats(end_time, file_sum);
-
-#endif
-   }
-
+   _mcrypt_time_show_stats(file_sum);
 
    return 0;
 }
@@ -500,7 +422,7 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
    word8 *ciphertext_old, *ciphertext_p_old;
    word32 *keyword;
    word32 *salt;
-   char local_algorithm[50], *command;
+   char local_algorithm[50], *command=NULL;
    char local_mode[50];
    char local_keymode[50];
    char local_salt[100];
@@ -509,13 +431,6 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
    int j;
    FILE *RTOF;
    FILE *FROMF;
-#ifdef HAVE_FTIME
-   struct timeb start_time;
-   struct timeb end_time;
-#else
-   time_t start_time;
-   time_t end_time;
-#endif
    size_t file_sum = 0;
 
 
@@ -525,6 +440,7 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
       RTOF = stdout;
 #ifdef ZIP
       if (bzipflag == TRUE || gzipflag == TRUE) {
+         err_info( "Will decompress input file.\n");
 	 clen = 10;
 	 if (fromfile != NULL)
 	    clen += strlen(fromfile);
@@ -557,20 +473,12 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
 	 perror("fopen");
 	 return 1;
       }
-      if (read_lock(fileno((FILE *) FROMF)) == -1) {
-	 err_crit("Unable to lock file. Try --nolock\n");
-	 return 1;
-      }
 
       RTOF = fopen(tofile, "wb");
       if (RTOF == NULL) {
 	 perror("fopen");
 	 return -1;
       }
-      if (write_lock(fileno((FILE *) RTOF)) == -1) {
-	 err_crit("Unable to lock file. Try --nolock\n");
-	 return -1;
-      }
 
    }
 
@@ -612,18 +520,7 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
    ciphertext = _mcrypt_malloc(blocksize * BUFFER_SIZE);
    ciphertext_old = _mcrypt_malloc(blocksize);
 
-   if (quiet == FALSE)
-      fprintf(stderr,
-	      _
-	      ("Algorithm: %s\nKeysize: %d\nMode: %s\nKeyword mode: %s\n"),
-	      algorithm, keysize, mode, keymode);
 
-
-
-   if (quiet <= TRUE && stream_flag == FALSE) {
-      sprintf(tmperr, _("File: %s\n"), fromfile);
-      err_warn(tmperr);
-   }
 /* Get key */
 
    if (key == NULL) {
@@ -659,12 +556,8 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
    fprintf(stderr, "\n");
 #endif
 
-#ifdef HAVE_FTIME
-   ftime(&start_time);
-#else
-   start_time = time(0);
-#endif
-
+   _mcrypt_start_timer();
+   
    j = mcrypt_generic_init(td, keyword, len, IV);
    if (j < 0) {
       mcrypt_perror(j);
@@ -952,20 +845,11 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
    }
 
 
-#ifdef HAVE_FTIME
-   ftime(&end_time);
-#else
-   end_time = time(0);
-#endif
-
-
-/* close and unlock files */
+/* close files */
 
    if (stream_flag == FALSE) {
       fflush(FROMF);
       fflush(RTOF);
-      unlock(fileno((FILE *) FROMF));
-      unlock(fileno((FILE *) RTOF));
    }
    fclose(RTOF);
    fclose(FROMF);
@@ -1040,30 +924,11 @@ decrypt_general(char *algorithm, char *fromfile, char *tofile, char *key)
    }
 
    mcrypt_generic_end(td);
+   _mcrypt_end_timer();
+   
+   print_enc_info( fromfile, tofile);
 
-   if (timer != FALSE) {
-#ifdef HAVE_FTIME
-      end_time.time *= 1000;
-      end_time.time += end_time.millitm;
-
-      start_time.time *= 1000;
-      start_time.time += start_time.millitm;
-
-      end_time.time -= start_time.time;
-
-      start_time.time = end_time.time;
-
-      end_time.millitm = end_time.time % 1000;
-
-      show_stats(end_time.time, file_sum);
-
-#else
-      end_time -= start_time;
-
-      show_stats(end_time, file_sum);
-
-#endif
-   }
+   _mcrypt_time_show_stats(file_sum);
 
    return 0;
 
@@ -1149,10 +1014,13 @@ int print_keylist(void)
    printf("pkdes\n");
    printf("s2k-simple-md5\n");
    printf("s2k-simple-sha1\n");
+   printf("s2k-simple-ripemd\n");
    printf("s2k-salted-md5\n");
    printf("s2k-salted-sha1\n");
+   printf("s2k-salted-ripemd\n");
    printf("s2k-isalted-md5\n");
    printf("s2k-isalted-sha1\n");
+   printf("s2k-isalted-ripemd\n");
 
    return 0;
 }
@@ -1169,68 +1037,10 @@ int print_hashlist(void)
       if (tmpc != NULL) {
 	 strcpy(tmp, tmpc);
 	 free(tmpc);
-	 _tolow(tmp, strlen(tmp));
+	 mcrypt_tolow(tmp, strlen(tmp));
 	 fprintf(stdout, "%s\n", tmp);
       }
    }
    return 0;
 }
 
-/* Accepts time elapsed in milliseconds and file size in bytes.
- */
-static void show_stats(time_t ms_time, size_t file_size)
-{
-   time_t time = ms_time / 1000;
-   time_t millitm = ms_time % 1000;
-   int show = 0;
-
-   fprintf(stderr, "Time needed: ");
-
-   if (time / 3600 > 0) {
-      fprintf(stderr, "%lu hours", time / 3600);
-      show = 1;
-   }
-
-   if ((time / 60) - ((time / 3600) * 60) > 0) {
-      if (show > 0)
-	 fprintf(stderr, ", ");
-      fprintf(stderr, "%lu minutes", (time / 60) - (time / 3600));
-      show = 1;
-   }
-
-   if (time - ((time / 60) + ((time / 3600) * 60) * 60) > 0) {
-      if (show > 0)
-	 fprintf(stderr, ", ");
-      fprintf(stderr, "%lu seconds", time - ((time / 60) + (time / 3600)));
-      show = 1;
-   }
-
-   if (millitm > 0) {
-      if (show > 0)
-	 fprintf(stderr, ", ");
-      fprintf(stderr, "%lu milliseconds", millitm);
-      show = 1;
-   }
-
-   if (show == 0)
-      fprintf(stderr, "Wow, no time counted!\n");
-   else {
-      fprintf(stderr, ".\n");
-
-      if (file_size > 1000000) {
-	 fprintf(stderr,
-		 _
-		 ("Encrypted %.3f MB, at a %.3f MB/sec rate (1 MB == 1000000 Bytes).\n"),
-		 (float) ((double) file_size / (double) 1000000),
-		 (float) ((((double) file_size / (double) 1000000) /
-			   (double) ms_time) * (double) 1000));
-      } else {			/* KB */
-	 fprintf(stderr,
-		 _
-		 ("Encrypted %.3f KB, at a %.3f KB/sec rate (1 KB == 1000 Bytes).\n"),
-		 (float) ((double) file_size / (double) 1000),
-		 (float) ((((double) file_size / (double) 1000) /
-			   (double) ms_time) * (double) 1000));
-      }
-   }
-}
