@@ -108,7 +108,10 @@
 # define DEFAULT_ALGORITHM "rijndael-128" /* The AES winner */
 
 #define DEFAULT_MODE "cbc"
+#define DEFAULT_PGP_MODE "ncfb"
+#define DEFAULT_PGP_ALGO "cast-128"
 #define DEFAULT_KEYMODE "mcrypt-sha1"
+#define DEFAULT_PGP_KEYMODE "s2k-isalted-sha1"
 #define HASH_ALGORITHM "sha1"
 
 #define ENCRYPT 0
@@ -140,18 +143,6 @@
  typedef unsigned char word8;
 #endif
 
-/* originally was defined in 3way.c */
-#ifndef HAVE_DEV_RANDOM
-# define RAND32 (word32) ((word32)rand() << 17 ^ (word32)rand() << 9 ^ rand())
-# define SRAND32 RAND32
-# define RAND8 (word8) ((word8)rand() << 4 ^ (word8)rand() << 2 ^ rand())
-# define SRAND8 RAND8
-#else
-# define RAND32 get_rand32()
-# define RAND8 get_rand8()
-# define SRAND32 get_safe_rand32()
-# define SRAND8 get_safe_rand8()
-#endif
 
 #ifndef HAVE_STRLCPY
 # define strlcpy(x, y, s) strncpy(x, y, s); x[s-1]='\0'
