@@ -1,5 +1,5 @@
 /*
- *    Copyright (C) 1998,1999,2000,2001 Nikos Mavroyanopoulos
+ *    Copyright (C) 1998,1999,2000,2001,2002 Nikos Mavroyanopoulos
  *
  *    This program is free software; you can redistribute it and/or modify
  *    it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 
 /* Some functions that didn't fit elsewhere */
 
-/* $Id: extra.c,v 1.12 2001/09/16 18:16:39 nmav Exp $ */
+/* $Id: extra.c,v 1.20 2002/05/29 11:07:43 nmav Exp $ */
 
 #ifndef DEFINES_H
 #define DEFINES_H
@@ -27,7 +27,6 @@
 
 #include "extra.h"
 #include <xmalloc.h>
-#include "locks.h"
 #include <errors.h>
 #ifndef NO_GETPASS
 # include <getpass.h>
@@ -38,8 +37,14 @@
 #include <bits.h>
 #include "mcrypt_int.h"
 
+#ifdef HAVE_FTIME
+# include <sys/timeb.h>
+#else
+# include <time.h>
+#endif
+
 static char rcsid[] =
-    "$Id: extra.c,v 1.12 2001/09/16 18:16:39 nmav Exp $";
+    "$Id: extra.c,v 1.20 2002/05/29 11:07:43 nmav Exp $";
 
 #ifndef NO_GETPASS
 	#define FREE_STR(x) \
@@ -49,9 +54,15 @@ static char rcsid[] =
 		Bzero(x, strlen(x))
 #endif
 
-
+extern int openpgp;
+extern int bare_flag;
+extern int quiet;
 extern int double_check;
 extern char *outfile;
+extern char* algorithm;
+extern int keysize;
+extern char* mode;
+extern char* keymode;
 extern int stream_flag;
 extern int bare_flag;
 extern int cleanDelete;
@@ -61,7 +72,7 @@ extern int hash_algorithm;
 char *my_getpass(char *prt)
 {
 	char *atmp;
-	char *ztmp = _secure_mcrypt_malloc(MAX_KEY_LEN);
+	static char ztmp[MAX_KEY_LEN];
 	char *btmp;
 	char string_tmp[200];
 
@@ -69,7 +80,6 @@ char *my_getpass(char *prt)
 	
 	atmp = _mcrypt_getpass(prt);
 	if (atmp==NULL) {
-		_secure_mcrypt_free(ztmp, MAX_KEY_LEN);
 		return NULL;
 	}
 	
@@ -77,14 +87,10 @@ char *my_getpass(char *prt)
 	Bzero(ztmp, MAX_KEY_LEN);
 	strlcpy(ztmp, atmp, MAX_KEY_LEN);
 
-	FREE_STR(atmp);
-
-	strcpy(string_tmp, _("Re-"));
-	strcat(string_tmp, prt);
+	strcpy(string_tmp, prt);
 
 	btmp = _mcrypt_getpass(string_tmp);
 	if (btmp==NULL) {
-		_secure_mcrypt_free(ztmp, MAX_KEY_LEN);
 		return NULL;
 	}
 
@@ -93,18 +99,14 @@ char *my_getpass(char *prt)
 			_
 			("Keywords do not match or they are too long.\n"));
 
-		FREE_STR( btmp);
-		
-		_secure_mcrypt_free(ztmp, MAX_KEY_LEN);
 		return NULL;
 	}
-	
-	FREE_STR( btmp);
+	fprintf(stderr, "\n");
 	
 	return ztmp;
 }
 
-char *get_password(int numofchar, int mode, unsigned int *len, void *salt)
+char *get_password( int mode, unsigned int *len)
 {
 
 	char *tmp = NULL;
@@ -129,6 +131,7 @@ char *get_password(int numofchar, int mode, unsigned int *len, void *salt)
 			tmp = my_getpass(msg);
 		}
 	}
+	
 	if (tmp == NULL)
 		return NULL;
 
@@ -296,7 +299,7 @@ void *read_iv(FILE * fstream, int ivsize)
 
 }
 
-void _tolow(char *str, int size)
+void mcrypt_tolow(char *str, int size)
 {
 	int i;
 
@@ -372,7 +375,7 @@ int write_file_head(FILE * filedes, char *algorithm, char *mode,
 	if (buf!=NULL) {
 		strcpy(tmp, buf);
 		free(buf);
-		_tolow(tmp, strlen(tmp));
+		mcrypt_tolow(tmp, strlen(tmp));
 	} else
 		return 1;
 		
@@ -497,11 +500,7 @@ char **read_key_file(char *file, int *num)
 			_("Keyfile could not be opened. Ignoring it.\n"));
 		return NULL;
 	}
-	if (read_lock(fileno((FILE *) FROMF)) == -1) {
-		fprintf(stderr,
-			_("Keyfile could not be locked. Ignoring it.\n"));
-		return NULL;
-	}
+
 	while (fgets(keyword, MAX_KEY_LEN, FROMF) != NULL) {
 		x++;
 		keys = _mcrypt_realloc(keys, x * sizeof(char *));
@@ -515,7 +514,6 @@ char **read_key_file(char *file, int *num)
 
 	*num = x;
 
-	unlock(fileno((FILE *) FROMF));
 	return keys;
 
 }
@@ -612,3 +610,176 @@ void Bzero(void *s, size_t n)
 # endif
 #endif
 }
+
+
+/* This function will check the first by of the file and will tell
+ * if this is an openpgp file or not.
+ */
+
+void test_file_headers( char* file) {
+unsigned char x;
+
+	if (file==NULL) {
+		x = getc( stdin);
+		if (ungetc( x, stdin)==EOF) {
+			err_quit(_("Could not push character back to stream\n"));
+		}
+	} else {
+		FILE* fd;
+		
+		fd = fopen( file, "rb");
+		x = getc( fd);
+		fclose(fd);
+	}
+
+	if (x==0) { /* mcrypt file format */
+		openpgp = 0;
+		return;
+	}
+	
+	if ( !(!x & 0x80) && bare_flag == FALSE) {
+		if (openpgp==0) {
+			err_warn(_("An OpenPGP encrypted file has been detected.\n"));
+		}
+		openpgp = 1;
+	}
+	return;
+}
+
+
+void print_enc_info(const char* infile, const char* outfile) {
+char tmperr[256];
+
+   if (quiet == FALSE) {
+      fprintf(stderr,
+	      _
+	      ("Algorithm: %s\nKeysize: %d\nMode: %s\nKeyword mode: %s\n"),
+	      algorithm, keysize, mode, keymode);
+   }
+   
+   if ( stream_flag == FALSE) {
+      sprintf(tmperr, _("Input File: %s\n"), infile);
+      err_info(tmperr);
+
+      sprintf(tmperr, _("Output File: %s\n"), outfile);
+      err_info(tmperr);
+   }
+
+   err_info("\n");
+
+}
+
+extern int timer; /* mcrypt.c */
+
+#ifdef HAVE_FTIME
+ static struct timeb start_time;
+ static struct timeb end_time;
+#else
+ static time_t start_time;
+ static time_t end_time;
+#endif
+
+void _mcrypt_start_timer(void) {
+
+#ifdef HAVE_FTIME
+ ftime(&start_time);
+#else
+ start_time = time(0);
+#endif
+}
+
+void _mcrypt_end_timer(void) {
+#ifdef HAVE_FTIME
+ ftime(&end_time);
+#else
+ end_time = time(0);
+#endif
+
+
+}
+
+/* Accepts time elapsed in milliseconds and file size in bytes.
+ */
+static void show_stats2(time_t ms_time, size_t file_size)
+{
+   time_t time = ms_time / 1000;
+   time_t millitm = ms_time % 1000;
+   int show = 0;
+
+   fprintf(stderr, _("Time needed: "));
+
+   if (time / 3600 > 0) {
+      fprintf(stderr, _("%lu hours"), time / 3600);
+      show = 1;
+   }
+
+   if ((time / 60) - ((time / 3600) * 60) > 0) {
+      if (show > 0)
+	 fprintf(stderr, ", ");
+      fprintf(stderr, _("%lu minutes"), (time / 60) - (time / 3600));
+      show = 1;
+   }
+
+   if (time - ((time / 60) + ((time / 3600) * 60) * 60) > 0) {
+      if (show > 0)
+	 fprintf(stderr, ", ");
+      fprintf(stderr, _("%lu seconds"), time - ((time / 60) + (time / 3600)));
+      show = 1;
+   }
+
+   if (millitm > 0) {
+      if (show > 0)
+	 fprintf(stderr, ", ");
+      fprintf(stderr, _("%lu milliseconds"), millitm);
+      show = 1;
+   }
+
+   if (show == 0)
+      fprintf(stderr, _("Wow, no time counted!\n"));
+   else {
+      fprintf(stderr, ".\n");
+
+      if (file_size > 1000000) {
+	 fprintf(stderr,
+		 _
+		 ("Processed %.3f MB, at a %.3f MB/sec rate (1 MB == 1000000 Bytes).\n\n"),
+		 (float) ((double) file_size / (double) 1000000),
+		 (float) ((((double) file_size / (double) 1000000) /
+			   (double) ms_time) * (double) 1000));
+      } else {			/* KB */
+	 fprintf(stderr,
+		 _
+		 ("Processed %.3f KB, at a %.3f KB/sec rate (1 KB == 1000 Bytes).\n\n"),
+		 (float) ((double) file_size / (double) 1000),
+		 (float) ((((double) file_size / (double) 1000) /
+			   (double) ms_time) * (double) 1000));
+      }
+   }
+}
+
+void _mcrypt_time_show_stats( size_t file_sum) {
+   if (timer != FALSE && file_sum > 0) {
+#ifdef HAVE_FTIME
+      end_time.time *= 1000;
+      end_time.time += end_time.millitm;
+
+      start_time.time *= 1000;
+      start_time.time += start_time.millitm;
+
+      end_time.time -= start_time.time;
+
+      start_time.time = end_time.time;
+
+      end_time.millitm = end_time.time % 1000;
+
+      show_stats2(end_time.time, file_sum);
+
+#else
+      end_time -= start_time;
+
+      show_stats2(end_time, file_sum);
+
+#endif
+
+   }
+}
