@@ -16,7 +16,7 @@
  *    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: keys.c,v 1.3 2001/09/02 20:20:43 nmav Exp $ */
+/* $Id: keys.c,v 1.6 2002/05/26 18:20:30 nmav Exp $ */
 
 #ifndef DEFINES_H
 #define DEFINES_H
@@ -27,7 +27,40 @@
 #include <errors.h>
 #include <ufc_crypt.h>
 
-static char rcsid[] = "$Id: keys.c,v 1.3 2001/09/02 20:20:43 nmav Exp $";
+static char rcsid[] = "$Id: keys.c,v 1.6 2002/05/26 18:20:30 nmav Exp $";
+
+int _mcrypt_pgp_conv_keymode( const char* keymode, keygenid* km, hashid *halg) {
+	if (strcasecmp(keymode, "s2k-simple-md5")==0) {
+		*km=KEYGEN_S2K_SIMPLE; 
+		*halg=MHASH_MD5;
+	} else if (strcasecmp(keymode, "s2k-simple-sha1")==0) {
+		*km=KEYGEN_S2K_SIMPLE; 
+		*halg=MHASH_SHA1;
+	} else if (strcasecmp(keymode, "s2k-simple-ripemd")==0) {
+		*km=KEYGEN_S2K_SIMPLE; 
+		*halg=MHASH_RIPEMD160;
+	} else if (strcasecmp(keymode, "s2k-salted-md5")==0) {
+		*km=KEYGEN_S2K_SALTED; 
+		*halg=MHASH_MD5;
+	} else if (strcasecmp(keymode, "s2k-salted-sha1")==0) {
+		*km=KEYGEN_S2K_SALTED; 
+		*halg=MHASH_SHA1;
+	} else if (strcasecmp(keymode, "s2k-salted-ripemd")==0) {
+		*km=KEYGEN_S2K_SALTED; 
+		*halg=MHASH_RIPEMD160;
+	} else if (strcasecmp(keymode, "s2k-isalted-md5")==0) {
+		*km=KEYGEN_S2K_ISALTED; 
+		*halg=MHASH_MD5;
+	} else if (strcasecmp(keymode, "s2k-isalted-sha1")==0) {
+		*km=KEYGEN_S2K_ISALTED; 
+		*halg=MHASH_SHA1;
+	} else if (strcasecmp(keymode, "s2k-isalted-ripemd")==0) {
+		*km=KEYGEN_S2K_ISALTED; 
+		*halg=MHASH_RIPEMD160;
+	} else return -1;
+
+	return 0;
+}
 
 int mcrypt_gen_key ( char* keymode, void* keyword, int keysize, 
 	void* salt, int saltsize, word8* password, int plen) {
@@ -43,12 +76,7 @@ int mcrypt_gen_key ( char* keymode, void* keyword, int keysize,
 	if (strcasecmp(keymode, "pkdes")==0) id=KEYGEN_PKDES;
 	if (strcasecmp(keymode, "mcrypt-sha1")==0) {id=KEYGEN_MCRYPT; algo=MHASH_SHA1;}
 	if (strcasecmp(keymode, "mcrypt-md5")==0) {id=KEYGEN_ASIS; algo=MHASH_MD5;}
-	if (strcasecmp(keymode, "s2k-simple-md5")==0) {id=KEYGEN_S2K_SIMPLE; algo=MHASH_MD5;}
-	if (strcasecmp(keymode, "s2k-simple-sha1")==0) {id=KEYGEN_S2K_SIMPLE; algo=MHASH_SHA1;}
-	if (strcasecmp(keymode, "s2k-salted-md5")==0) {id=KEYGEN_S2K_SALTED; algo=MHASH_MD5;}
-	if (strcasecmp(keymode, "s2k-salted-sha1")==0) {id=KEYGEN_S2K_SALTED; algo=MHASH_SHA1;}
-	if (strcasecmp(keymode, "s2k-isalted-md5")==0) {id=KEYGEN_S2K_ISALTED; algo=MHASH_MD5;}
-	if (strcasecmp(keymode, "s2k-isalted-sha1")==0) {id=KEYGEN_S2K_ISALTED; algo=MHASH_SHA1;}
+	_mcrypt_pgp_conv_keymode( keymode, &id, &algo);
 
 	if (icrypt==0) {
 		data.hash_algorithm[0] = algo;
@@ -82,7 +110,7 @@ void *
 	keyword = _mcrypt_calloc(1, keysize);
 
 	if (key == NULL) {	/* key was not specified in the command line */
-		tmp = get_password(keysize, enc_mode, len, salt);
+		tmp = get_password( enc_mode, len);
 		if (tmp == NULL)
 			return NULL;
 
@@ -90,11 +118,7 @@ void *
 			*len = MAX_KEY_LEN;
 		memmove(tmpkey, tmp, *len);
 
-#ifndef NO_GETPASS
-		_secure_mcrypt_free(tmp, *len);
-#else
 		Bzero(tmp, strlen(tmp));
-#endif
 	} else {		/* Key was given on the command line */
 		if (*len > MAX_KEY_LEN)
 			*len = MAX_KEY_LEN;
@@ -132,7 +156,6 @@ void *
 #endif
 
 	Bzero( tmpkey, MAX_KEY_LEN);
-	_mcrypt_free( tmpkey);
 	
 	return keyword;
 
@@ -147,11 +170,14 @@ int _which_algo ( char* keymode) {
 	if (strcasecmp(keymode, "mcrypt-sha1")==0) return 0;
 	if (strcasecmp(keymode, "mcrypt-md5")==0) return 0;
 	if (strcasecmp(keymode, "s2k-simple-md5")==0) return 0;
+	if (strcasecmp(keymode, "s2k-simple-ripemd")==0) return 0;
 	if (strcasecmp(keymode, "s2k-simple-sha1")==0) return 0;
 	if (strcasecmp(keymode, "s2k-salted-md5")==0) return 0;
 	if (strcasecmp(keymode, "s2k-salted-sha1")==0) return 0;
+	if (strcasecmp(keymode, "s2k-salted-ripemd")==0) return 0;
 	if (strcasecmp(keymode, "s2k-isalted-md5")==0) return 0;
 	if (strcasecmp(keymode, "s2k-isalted-sha1")==0) return 0;
+	if (strcasecmp(keymode, "s2k-isalted-ripemd")==0) return 0;
 
 	return -1;
 }
