@@ -19,7 +19,7 @@
  */
 
 
-/* $Id: mcrypt.c,v 1.20 2002/03/07 12:24:15 nmav Exp $ */
+/* $Id: mcrypt.c,v 1.30 2002/05/26 10:26:18 nmav Exp $ */
 
 #ifndef DEFINES_H
 #define DEFINES_H
@@ -28,13 +28,13 @@
 #include <mcrypt_int.h>
 #include <xmalloc.h>
 #include <extra.h>
-#include <locks.h>
 #include <keys.h>
 #include <random.h>
 #include <gaa.h>
 #include <errors.h>
 #include <environ.h>
 #include <popen.h>
+#include <openpgp.h>
 
 #ifdef HAVE_FTIME
 # include <sys/timeb.h>
@@ -43,7 +43,7 @@
 #endif
 
 static char rcsid[] =
-    "$Id: mcrypt.c,v 1.20 2002/03/07 12:24:15 nmav Exp $";
+    "$Id: mcrypt.c,v 1.30 2002/05/26 10:26:18 nmav Exp $";
 
 char tmperr[128];
 unsigned int stream_flag = FALSE;
@@ -53,7 +53,6 @@ char *algorithm = NULL;
 char *hash = NULL;
 int keysize;
 int nodelete = FALSE;		/* Delete by default */
-int nolock = FALSE;		/* Use locks by default */
 int noiv = FALSE;
 int hash_algorithm = MHASH_CRC32;
 int noecho = TRUE;		/* Echo asterisks by default */
@@ -69,94 +68,27 @@ int bzipflag = FALSE;
 #endif
 int flush = FALSE;
 int timer = FALSE;
+int openpgp = FALSE;
+int openpgp_z = 0; /* default is no compression */
 char *outfile = 0, *tmpc;
 char *algorithms_directory = NULL;
 char *modes_directory = NULL;
+char* program_name = NULL;
 
 void usage(void)
 {
-
-   fprintf(stdout,
-	   _
-	   ("Usage: mcrypt [-dLFus87bhvrx] [-f keyfile] [-k key1 key2 ...] [-m mode] [-o keymode] [-s keysize] [-a algorithm] [-c config_file] [file ...]\n\n"));
-
-   fprintf(stdout,
-	   _
-	   ("mcrypt encrypts/decrypts files using several symmetric algorithms.\n"));
-   fprintf(stdout,
-	   _("Legal parameters are:\n" "\n"
-	     " -d --decrypt                 decrypts\n"
-	     " -c --config FILENAME         Use the configuration file FILENAME.\n"
-	     "                              If not specified the .mcryptrc in your home\n"
-	     "                              directory is used.\n"
-	     " -a --algorithm ALGORITHM     specify the encryption and decryption algorithm.\n"
-	     "                              Use the --list parameter to see the supported\n"
-	     "                              algorithms.\n"
-	     " --algorithms_directory DIR   specify the directory where the algorithms are.\n"));
-   fprintf(stdout,
-	   _
-	   (" -m --mode MODE               specify the mode. Use the --list parameter\n"
-	    "	                        to see the supported modes.\n"
-	    " --modes_directory DIR        specify the directory where the modes are.\n"
-	    " -k --key KEY1, KEY2, ...     keyword(s) to use.\n"
-	    " -f --keyfile FILENAME        file to read the keyword from.\n"
-	    " -o --keymode MODE            specify the keyword mode. Use the --keymodeslist\n"
-	    "                               parameter to view all modes.\n"
-	    " -h --hash ALGORITHM          specify the hash algorithm to be used, in order to compute\n"
-	    "                               the checksum. Use the --hashlist parameter to\n"
-	    "                               view all algorithms.\n"
-	    " -s --keysize SIZE            Algorithm's key size. The default is the maximum\n"
-	    "                               supported by the algorithm.\n"
-	    " -u --unlink                  unlink the input file after encryption/decryption\n"
-	    " -L --license                 displays license information.\n"
-	    " -q --quiet                   suppress some non critical warnings.\n"
-	    " -l --doublecheck             Double check for passphrase even if decrypting.\n"
-	    "    --echo                    Echo asterisks when entering the password.\n"
-	    "    --list                    Lists the supported algorithms and modes.\n"
-	    "    --keymodeslist            Lists the supported key modes.\n"
-	    "    --hashlist                Lists the supported hash algorithms.\n"
-	    "    --flush                   Immediately flush the output.\n"
-	    "    --nodelete                Does not delete the output file even if decryption\n"
-	    "				fails.\n"
-	    " -V --verbose                 more information is displayed.\n"
-	    " -b --bare                    Do not keep algorithm information in the encrypted\n"
-	    "                              file.\n"
-	    " -F --force                   forces output to stdout.\n"
-	    " -t --time                    Prints timing information.\n"));
-
-#ifdef HAVE_DEV_RANDOM
-   fprintf(stdout,
-	   _(" -r --random                  Use real random data.\n"));
-#endif
-
-#ifdef GZIP
-   fprintf(stdout,
-	   _
-	   (" -z --gzip                    use gzip to compress files before encryption.\n"));
-#endif
-
-#ifdef BZIP2
-   fprintf(stdout,
-	   _
-	   (" -p --bzip2                   use bzip2 to compress files before encryption.\n"));
-#endif
-
-#ifndef NO_FCNTL_LOCK
-   fprintf(stdout,
-	   _(" -n --nolock                  Do not lock files.\n"));
-#endif
-
+	gaa_help();
    fprintf(stdout, _("\n"
-		     "    --help                    prints this help\n"
-		     " -v --version                 prints the version number\n"
-		     "\n"
 		     "Report bugs to mcrypt-dev@lists.hellug.gr.\n\n"));
+	exit(0);
+
 }
 
+static char* remove_suffix( char* filename);
 void mcrypt_license()
 {
    fprintf(stdout,
-	   _("\nCopyright (C) 1998-2001 Nikos Mavroyanopoulos\n"
+	   _("\nCopyright (C) 1998-2002 Nikos Mavroyanopoulos\n"
 	     "This program is free software; you can redistribute it and/or modify \n"
 	     "it under the terms of the GNU General Public License as published by \n"
 	     "the Free Software Foundation; either version 2 of the License, or \n"
@@ -181,7 +113,7 @@ int check_hash_algo(char *chain)
       if (y != NULL) {
 	 strcpy(tmp, y);
 	 free(y);
-	 _tolow(tmp, strlen(tmp));
+	 mcrypt_tolow(tmp, strlen(tmp));
 	 if (strcmp(tmp, chain) == 0) {
 	    return i;
 	 }
@@ -206,10 +138,49 @@ void mcrypt_version()
 	   LIBMCRYPT_VERSION);
    fprintf(stderr,
 	   _
-	   ("Copyright (C) 1998-2001 Nikos Mavroyanopoulos (nmav@hellug.gr)\n"));
+	   ("Copyright (C) 1998-2002 Nikos Mavroyanopoulos (nmav@gnutls.org)\n"));
 
 }
 
+/* Checks if the given key size is supported by the algorithm
+ */
+static void check_keysize( int keysize) {
+      int *siz, i, y, x;
+
+      if ( keysize >
+	  mcrypt_module_get_algo_key_size(algorithm,
+					  algorithms_directory)) {
+	 err_quit(_
+		  ("The specified key size is too large for this algorithm.\n"));
+      }
+
+      siz =
+	  mcrypt_module_get_algo_supported_key_sizes(algorithm,
+						     algorithms_directory,
+						     &i);
+      x = 0;
+      if (siz == NULL || i == 0)
+	 x = 1;			/* we have already checked if it is
+				   * less than the maximum supported
+				   * by the algorithm
+				 */
+      else {
+	 for (y = 0; y < i; y++) {
+	    if (siz[y] == keysize) {
+	       x = 1;
+	       break;
+	    }
+	 }
+	 free(siz);
+      }
+
+      if (x == 0) {
+	 err_quit(_
+		  ("The specified key size not valid for this algorithm.\n"));
+      }
+}
+
+
 /*                              The main    */
 int main(int argc, char **argv)
 {
@@ -238,13 +209,14 @@ int main(int argc, char **argv)
    textdomain(PACKAGE);
 #endif
 
+   program_name = argv[0];
+   
    if ((gaa(argc, argv, &info)) != -1) {
       err_quit(_
 	       ("Error in the arguments. Use the --help parameter for more info.\n"));
    }
    if (info.config_file != NULL) {
-      cfile = _mcrypt_malloc(strlen(info.config_file));
-      strcpy(cfile, info.config_file);
+      cfile = _mcrypt_strdup( info.config_file);
       if (check_file(cfile) != 1) {
 	 err_crit(_("Warning: Cannot access config file.\n"));
       }
@@ -252,12 +224,12 @@ int main(int argc, char **argv)
 #ifdef HAVE_GETPWUID
       cfile = get_cfile(getuid(), ".mcryptrc");
 #else
-      cfile = _mcrypt_malloc(11);
-      strcpy(cfile, ".mcryptrc");
+      cfile = _mcrypt_strdup( ".mcryptrc");
 #endif
    }
 
-   /* If config file exists */
+   /* If config file exists 
+    */
    if (check_file(cfile) == 1) {
       if ((gaa_file(cfile, &info)) != -1) {
 	 /* gets options from file 'config' */
@@ -296,22 +268,24 @@ int main(int argc, char **argv)
 	       ("Error in the arguments. Use the --help parameter for more info.\n"));
    }
 /* Examine how we were called */
-   if (strstr(argv[0], "decrypt") != NULL) {
-      Bzero(argv[0], strlen(argv[0]));
-      strcpy(argv[0], "mdecrypt");
+   if (strstr(program_name, "decrypt") != NULL) {
+      Bzero(program_name, strlen(program_name));
+      strcpy(program_name, "mdecrypt");
       din = TRUE;
       ein = FALSE;
    } else {
-      if (strlen(argv[0]) > 6) {
-	 Bzero(argv[0], strlen(argv[0]));
-	 strcpy(argv[0], "mcrypt");
+      if (strlen(program_name) > 6) {
+	 Bzero(program_name, strlen(program_name));
+	 strcpy(program_name, "mcrypt");
       }
       din = FALSE;
       ein = TRUE;		/* It will change by the arguments */
    }
 
 
-   /* File pointers are as much as the file arguments */
+
+   /* File pointers are as much as the file arguments 
+    */
 
    if ((info.size) * sizeof(char *) > 0) {
       file = _mcrypt_malloc((info.size) * sizeof(char *));
@@ -331,17 +305,16 @@ int main(int argc, char **argv)
 	       ("Warning: This system does not support real random data.\n"));
 #endif
    }
+   
    force = info.force;
    bare_flag = info.bare_flag;
    unlink_flag = info.unlink_flag;
    quiet = info.quiet;
-   nolock = info.nolock;
    noecho = info.noecho;
    noiv = info.noiv;
    double_check = info.double_check;
    flush = info.flush;
    nodelete = info.nodelete;
-   keysize = info.keysize;
    timer = info.timer;
 
    algorithms_directory = info.algorithms_directory;
@@ -354,17 +327,15 @@ int main(int argc, char **argv)
    if (info.kmode != NULL) {
       keymode = info.kmode;
    } else {
-      keymode = strdup(DEFAULT_KEYMODE);	/* Default */
+      keymode = DEFAULT_KEYMODE;	/* Default */
    }
 
    if (info.hash != NULL) {
-      hash = _mcrypt_malloc(strlen(info.hash));
-      strcpy(hash, info.hash);
+      hash = _mcrypt_strdup( info.hash);
       hash_algorithm = check_hash_algo(hash);
 
    } else {
-      hash = _mcrypt_malloc(strlen(HASH_ALGORITHM) + 1);
-      strcpy(hash, HASH_ALGORITHM);
+      hash = _mcrypt_strdup( HASH_ALGORITHM);
       hash_algorithm = check_hash_algo(hash);
    }
 
@@ -397,16 +368,26 @@ int main(int argc, char **argv)
    }
 
    if (info.mode != NULL) {
-      mode = _mcrypt_malloc(strlen(info.mode) + 1);
-      strcpy(mode, info.mode);
+      mode = info.mode;
    } else {
-      mode = strdup(DEFAULT_MODE);
+      mode = DEFAULT_MODE;
+   }
+
+   openpgp = info.openpgp;
+   if (openpgp!=0) {
+   	mode = DEFAULT_PGP_MODE;
+   	if (strncasecmp( "s2k", keymode, 3)!=0) {
+   		keymode = DEFAULT_PGP_KEYMODE;
+   	}
+   }
+   openpgp_z = info.openpgp_z;
+   if ( openpgp_z < 0) {
+   	err_quit(_("Illegal compression level\n"));
    }
 
    if (info.algorithm != NULL) {
       used_algo = TRUE;
-      algorithm = _mcrypt_malloc(strlen(info.algorithm) + 1);
-      strcpy(algorithm, info.algorithm);
+      algorithm = _mcrypt_strdup( info.algorithm);
 
       i = mcrypt_module_is_block_algorithm(algorithm,
 					   algorithms_directory);
@@ -419,8 +400,7 @@ int main(int argc, char **argv)
       if (i == 0) {
 	 y = mcrypt_module_is_block_algorithm_mode(mode, modes_directory);
 	 if (y >= 0) {
-	    mode = _mcrypt_realloc(mode, strlen("stream") + 1);
-	    strcpy(mode, "stream");
+	    mode = "stream";
 	 } else {
 	    fprintf(stderr, _("Error in mode '%s'.\n"), mode);
 	    exit(1);
@@ -428,10 +408,10 @@ int main(int argc, char **argv)
       }
 
    } else {
-      algorithm = strdup(DEFAULT_ALGORITHM);
+      algorithm = DEFAULT_ALGORITHM;
    }
 
-   if (info.mode != NULL) {
+   if ( mode != NULL) {
       y = mcrypt_module_is_block_algorithm_mode(mode, modes_directory);
       if (y < 0) {
 	 fprintf(stderr, _("Error in mode '%s'.\n"), mode);
@@ -439,42 +419,12 @@ int main(int argc, char **argv)
       }
    }
 
-   if (info.keysize != 0) {
-      int *siz;
+   keysize = info.keysize;
+   if ( keysize != 0) {
+	check_keysize (keysize);
 
-      if (info.keysize >
-	  mcrypt_module_get_algo_key_size(algorithm,
-					  algorithms_directory)) {
-	 err_quit(_
-		  ("The specified key size is too large for this algorithm.\n"));
-      }
-
-      siz =
-	  mcrypt_module_get_algo_supported_key_sizes(algorithm,
-						     algorithms_directory,
-						     &i);
-      x = 0;
-      if (siz == NULL || i == 0)
-	 x = 1;			/* we have already checked if it is
-				   * less than the maximum supported
-				   * by the algorithm
-				 */
-      else {
-	 for (y = 0; y < i; y++) {
-	    if (siz[y] == info.keysize) {
-	       x = 1;
-	       break;
-	    }
-	 }
-	 free(siz);
-      }
-
-      if (x == 0) {
-	 err_quit(_
-		  ("The specified key size not valid for this algorithm.\n"));
-      }
-
-   }
+   } else
+   	keysize = mcrypt_module_get_algo_key_size(algorithm, algorithms_directory);
 
    if (info.keylen != 0 && check_env() == FALSE) {
       err_warn(_
@@ -491,7 +441,8 @@ int main(int argc, char **argv)
 #endif
 
 
-/* Normal startup */
+/* Normal startup 
+ */
 
 
 /* ok now how many files were specified? */
@@ -503,7 +454,6 @@ int main(int argc, char **argv)
 
    file_count += (info.size);
 
-
    if (file_count == 0) {
       stream_flag = TRUE;
    }
@@ -538,7 +488,7 @@ int main(int argc, char **argv)
 	    sprintf(tmperr,
 		    _
 		    ("%s: %s is not a regular file. Skipping...\n"),
-		    argv[0], file[i]);
+		    program_name, file[i]);
 	    err_crit(tmperr);
 	    outfile = NULL;
 	    continue;		/* next */
@@ -549,6 +499,7 @@ int main(int argc, char **argv)
 
       /* Check how we were called */
       if (din == TRUE) {	/* decryption */
+      
 	 if (stream_flag != TRUE)
 	    dinfile = file[i];
 	 if ((isatty(fileno((FILE *) (stdin))) == 1)
@@ -556,11 +507,17 @@ int main(int argc, char **argv)
 	    sprintf(tmperr,
 		    _
 		    ("%s: Encrypted data will not be read from a terminal.\n"),
-		    argv[0]);
+		    program_name);
 	    err_crit(tmperr);
 	    err_quit(_
 		     ("Redirect the input instead.\nUse the --help parameter for more help.\n"));
 	 }
+
+         /* this will enable the pgp flag if it is a pgp file.
+          * or disable it otherwise.
+          */
+         test_file_headers( dinfile);
+
       } else {			/* encryption */
 	 if (stream_flag != TRUE)
 	    einfile = file[i];
@@ -569,7 +526,7 @@ int main(int argc, char **argv)
 	    sprintf(tmperr,
 		    _
 		    ("%s: Encrypted data will not be written to a terminal.\n"),
-		    argv[0]);
+		    program_name);
 	    err_crit(tmperr);
 	    err_quit(_
 		     ("Redirect the output instead.\nUse the --help parameter for more help.\n"));
@@ -581,37 +538,10 @@ int main(int argc, char **argv)
       if (stream_flag != TRUE) {
 
 	 if (din == TRUE) {
+	 	outfile = remove_suffix( dinfile);
+	 	if (outfile==NULL)
+	 		continue;
 
-	    y = strlen(dinfile);
-	    /* If the file has the .nc suffix, then remove it */
-
-	    if ((dinfile[y - 1] == 'c'
-		 && dinfile[y - 2] == 'n' && dinfile[y - 3] == '.')) {
-
-	       outfile = _mcrypt_calloc(y - 2, 1);
-	       strncpy(outfile, dinfile, y - 3);
-#ifdef HAVE_STAT
-	       /* But if it exists exit */
-	       if (check_file(outfile) != 0) {
-		  cleanDelete = FALSE;
-		  if (ask_overwrite(argv[0], outfile) == FALSE) {
-		     continue;
-		  } else {
-		     cleanDelete = TRUE;
-		  }
-	       } else {
-		  cleanDelete = TRUE;
-	       }
-#endif
-	    } else {		/* append .dc */
-
-	       cleanDelete = TRUE;
-	       outfile = _mcrypt_calloc(y + 5, 1);
-	       strncpy(outfile, dinfile, y);
-	       strcat(outfile, ".dc");
-
-
-	    }
 	 } else {		/* encryption- append .nc */
 	    outfile = _mcrypt_calloc(strlen(einfile) + 5 + 4, 1);
 	    strcpy(outfile, einfile);
@@ -620,7 +550,7 @@ int main(int argc, char **argv)
 	       sprintf(tmperr,
 		       _
 		       ("%s: file %s has the .nc suffix... skipping...\n"),
-		       argv[0], outfile);
+		       program_name, outfile);
 	       err_crit(tmperr);
 	       continue;
 	    }
@@ -637,7 +567,7 @@ int main(int argc, char **argv)
 	    /* But if it exists exit */
 	    if (check_file(outfile) != 0) {
 	       cleanDelete = FALSE;
-	       if (ask_overwrite(argv[0], outfile)
+	       if (ask_overwrite(program_name, outfile)
 		   == FALSE) {
 		  continue;
 	       } else {
@@ -651,15 +581,15 @@ int main(int argc, char **argv)
 
 	 }
       } else {			/* if streams */
-	 outfile = _mcrypt_malloc(7);
-	 strcpy(outfile, "stdout");
+	 outfile = NULL;
       }
 
 
 
 /* Decrypt */
       if (din == TRUE) {
-	 x = decrypt_general(algorithm, dinfile, outfile, keyword);
+	 if (openpgp!=0) x = pgp_decrypt_wrap( dinfile, outfile, keyword);
+	 else x = decrypt_general(algorithm, dinfile, outfile, keyword);
 
 	 if (x == 0) {
 	    if (stream_flag == FALSE) {
@@ -704,7 +634,8 @@ int main(int argc, char **argv)
 
 /* Encrypt */
       if (ein == TRUE) {
-	 x = encrypt_general(algorithm, einfile, outfile, keyword);
+	 if (openpgp!=0) x = pgp_encrypt_wrap( einfile, outfile, keyword);
+	 else x = encrypt_general(algorithm, einfile, outfile, keyword);
 
 	 if (x == 0) {
 	    if (stream_flag == FALSE) {
@@ -779,5 +710,48 @@ int make_mult(int num, int d)
    return i;
 }
 
+static char* remove_suffix( char* filename) {
+int y;
+char* outfile;
+	    
+    y = strlen(filename);
+    /* If the file has the .nc suffix, then remove it */
 
+    if ((filename[y - 1] == 'c'
+	 && filename[y - 2] == 'n' && filename[y - 3] == '.')) {
 
+       outfile = _mcrypt_calloc(y - 2, 1);
+       strncpy(outfile, filename, y - 3);
+
+    } else /* gpg files */ if ((filename[y - 1] == 'g'
+	 && filename[y - 2] == 'p' && filename[y - 3] == 'g'
+	 && filename[y - 4] == '.')) {
+
+	       	outfile = _mcrypt_calloc(y - 3, 1);
+		strncpy(outfile, filename, y - 4);
+    } else { /* append .dc */
+       err_warn(_("Unknown suffix. Will append '.dc'.\n"));
+
+       cleanDelete = TRUE;
+       outfile = _mcrypt_calloc(y + 5, 1);
+       strncpy(outfile, filename, y);
+       strcat(outfile, ".dc");
+
+    }
+
+#ifdef HAVE_STAT
+       /* But if it exists exit */
+       if (check_file(outfile) != 0) {
+	  cleanDelete = FALSE;
+	  if (ask_overwrite( program_name, outfile) == FALSE) {
+	     return NULL;
+	  } else {
+	     cleanDelete = TRUE;
+	  }
+       } else {
+	  cleanDelete = TRUE;
+       }
+#endif
+
+	return outfile;
+}
